﻿using HospitalContracts.BindingModels;
using HospitalContracts.BusinessLogicsContracts;
using HospitalContracts.SearchModels;
using HospitalContracts.StoragesContracts;
using HospitalContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HospitalBusinessLogic.BusinessLogics
{
    public class KurseLogic : IKurseLogic { 
    private readonly ILogger _logger;
    private readonly IKurseStorage _kurseStorage;
    public KurseLogic(ILogger<KurseLogic> logger, IKurseStorage kurseStorage)
    {
        _logger = logger;
        _kurseStorage = kurseStorage;
    }
    public bool Create(KurseBindingModel model)
    {
        CheckModel(model);
        if (_kurseStorage.Insert(model) == null)
        {
            _logger.LogWarning("Insert operation failed");
            return false;
        }
        return true;
    }

    public bool Delete(KurseBindingModel model)
    {
        CheckModel(model, false);
        _logger.LogInformation("Delete. Id:{Id}", model.Id);
        if (_kurseStorage.Delete(model) == null)
        {
            _logger.LogWarning("Delete operation failed");
            return false;
        }
        return true;
    }

    public KurseViewModel? ReadElement(KurseSearchModel model)
    {
        if (model == null)
        {
            throw new ArgumentNullException(nameof(model));
        }
        _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
        var element = _kurseStorage.GetElement(model);
        if (element == null)
        {
            _logger.LogWarning("ReadElement element not found");
            return null;
        }
        _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
        return element;
    }

    public List<KurseViewModel>? ReadList(KurseSearchModel? model)
    {
        _logger.LogInformation("ReadElement. Id:{ Id}", model?.Id);
        var list = model == null ? _kurseStorage.GetFullList() : _kurseStorage.GetFilteredList(model);
        if (list == null)
        {
            _logger.LogWarning("ReadList return null list");
            return null;
        }
        _logger.LogInformation("ReadList. Count:{Count}", list.Count);
        return list;
    }

    public bool Update(KurseBindingModel model)
    {
        CheckModel(model);
        if (_kurseStorage.Update(model) == null)
        {
            _logger.LogWarning("Update operation failed");
            return false;
        }
        return true;
    }

    private void CheckModel(KurseBindingModel model, bool withParams = true)
    {
        if (model == null)
        {
            throw new ArgumentNullException(nameof(model));
        }

        if (!withParams)
        {
            return;
        }

        if (model.CountInDay <= 0)
        {
            throw new ArgumentNullException("Количество приемов в день должно быть больше 0", nameof(model.CountInDay));
        }
        _logger.LogInformation("Kurse. KurseId:{Id}.CountInDay:{ CountInDay}", model.Id, model.CountInDay);
    }
}
}

