﻿using HospitalClientApp.Models;
using HospitalContracts.BindingModels;
using HospitalContracts.BusinessLogicsContracts;
using HospitalContracts.SearchModels;
using HospitalContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Hosting;
using System.Diagnostics;
using System.Diagnostics.Metrics;
using System.Reflection;
using System.Xml.Linq;

namespace HospitalClientApp.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;
        private readonly IReportLogic _report;


        public HomeController(ILogger<HomeController> logger, IReportLogic report) //IReportLogic report
        {
            _logger = logger;
            _report = report;
        }

        public IActionResult Index()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View();
        }

        [HttpGet]
        public IActionResult Enter()
        {
            return View();
        }

        [HttpPost]
        public void Enter(string login, string password)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password))
            {
                throw new Exception("Введите логин и пароль");
            }
            APIClient.Client = APIClient.GetRequest<ClientViewModel>($"api/client/login?login={login}&password={password}");
            if (APIClient.Client == null)
            {
                throw new Exception("Неверный логин/пароль");
            }
            Response.Redirect("Index");
        }
        [HttpGet]
        public IActionResult Privacy()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.Client);
        }
        [HttpPost]
        public void Privacy(string login, string password, string fio)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
            }
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
            {
                throw new Exception("Введите логин, пароль и ФИО");
            }
            APIClient.PostRequest("api/client/updatedata", new ClientBindingModel
            {
                Id = APIClient.Client.Id,
                ClientFIO = fio,
                Email = login,
                Password = password
            });

            APIClient.Client.ClientFIO = fio;
            APIClient.Client.Email = login;
            APIClient.Client.Password = password;
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Register()
        {
            return View();
        }

        [HttpPost]
        public void Register(string login, string password, string fio)
        {
            if (string.IsNullOrEmpty(login) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(fio))
            {
                throw new Exception("Введите логин, пароль и ФИО");
            }
            APIClient.PostRequest("api/client/register", new ClientBindingModel
            {
                ClientFIO = fio,
                Email = login,
                Password = password
            });
            Response.Redirect("Enter");
            return;
        }
        //ОТЧЕТ//
        [HttpGet]
        public IActionResult ReportWordExel()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<MedicinesViewModel>>($"api/main/getmedicineslist?clientId={APIClient.Client.Id}"));
        }
        [HttpPost]
        public void ReportWordExel(int[] Ids, string type)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }

            if (Ids.Length <= 0)
            {
                throw new Exception("Количество должно быть больше 0");
            }

            if (string.IsNullOrEmpty(type))
            {
                throw new Exception("Неверный тип отчета");
            }

            List<int> res = new List<int>();

            foreach (var item in Ids)
            {
                res.Add(item);
            }

            if (type == "docx")
            {
                APIClient.PostRequest("api/report/createreporttowordfile", new ReportBindingModel
                {
                    Ids = res,
                    FileName = "E:\\ReportsCourseWork\\wordfile.docx"
                });
                Response.Redirect("GetWordFile");
            }
            else
            {
                APIClient.PostRequest("api/report/createreporttoexcelfile", new ReportBindingModel
                {
                    Ids = res,
                    FileName = "E:\\ReportsCourseWork\\excelfile.xlsx"
                });
                Response.Redirect("GetExcelFile");
            }
        }

        [HttpGet]
        public IActionResult GetWordFile()
        {
            return new PhysicalFileResult("E:\\ReportsCourseWork\\wordfile.docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document");
        }
        public IActionResult GetPdfFile()
        {
            return new PhysicalFileResult("E:\\ReportsCourseWork\\pdffile.pdf", "application/pdf");
        }
        public IActionResult GetExcelFile()
        {
            return new PhysicalFileResult("E:\\ReportsCourseWork\\excelfile.xlsx", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        }

        [HttpGet]
        public IActionResult ReportPdf()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View("ReportPdf");
        }

        [HttpPost]
        public void ReportPdf(DateTime dateFrom, DateTime dateTo, string clientEmail)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            if (string.IsNullOrEmpty(clientEmail))
            {
                throw new Exception("Email пуст");
            }
            APIClient.PostRequest("api/report/CreateReportToPdfFile", new ReportBindingModel
            {
                DateFrom = dateFrom,
                DateTo = dateTo,
                ClientId = APIClient.Client.Id
            });
            APIClient.PostRequest("api/report/SendPdfToMail", new MailSendInfoBindingModel
            {
                MailAddress = clientEmail,
                Subject = "Отчет по рецептам (pdf)",
                Text = "Отчет по рецептам с " + dateFrom.ToShortDateString() + " до " + dateTo.ToShortDateString()
            });
            Response.Redirect("ReportPdf");
        }

        [HttpGet]
        public string GetRecipesReport(DateTime dateFrom, DateTime dateTo)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как суда попали? Суда вход только авторизованным");
            }
            List<ReportRecipesViewModel> result;
            try
            {
                result = _report.GetRecipes(new ReportBindingModel
                {
                    ClientId = APIClient.Client.Id,
                    DateFrom = dateFrom,
                    DateTo = dateTo
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания отчета");
                throw;
            }

            string table = "";
            table += $"<h2>Предварительный отчет</h2>";
            table += $"<table class=\"table table-bordered table-striped\"/>";
            table += "<thead class=\"u-custom-color-1 u-table-header u-table-header-1\">";
            table += "<tr style=\"height: 31px\">";
            table += $"<th class=\"u-border-1 u-border-grey-50 u-table-cell\">Дата создания</th>";
            table += $"<th class=\"u-border-1 u-border-grey-50 u-table-cell\">Дозировка</th>";
            table += $"<th class=\"u-border-1 u-border-grey-50 u-table-cell\">Способ применения</th>";
            table += "</tr>";
            table += "</thead>";
            foreach (var report in result)
            {
                bool IsDate = true;
                if (report.Date.ToShortDateString() == "01.01.0001")
                {
                    IsDate = false;
                }
                table += "<tbody class=\"u-table-body\">";
                table += "<tr style=\"height: 75px\">";
                table += $"<td class=\"u-border-1 u-border-grey-40 u-border-no-left u-border-no-right u-table-cell\">{(IsDate is true ? report.Date.ToShortDateString() : string.Empty)}</td>";
                table += $"<td class=\"u-border-1 u-border-grey-40 u-border-no-left u-border-no-right u-table-cell\">{report.Dose}</td>";
                table += $"<td class=\"u-border-1 u-border-grey-40 u-border-no-left u-border-no-right u-table-cell\">{report.ModeofApplication}</td>";
                table += "</tr>";
                table += "</tbody>";
            }
            table += "</table>";
            return table;
        }
        /// <summary>
        /// ПРОЦЕДУРЫ
        /// </summary>
        /// <returns></returns>
        public IActionResult ListProcedures()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<ProceduresViewModel>>($"api/main/GetProcedureList?ClientId={APIClient.Client.Id}"));
        }
        public IActionResult CreateProcedure(int? id)
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Medicines = APIClient.GetRequest<List<MedicinesViewModel>>("api/main/getmedicineslist");
            if (!id.HasValue)
            {
                return View(new ProceduresViewModel());
            }
            var model = APIClient.GetRequest<ProceduresViewModel?>($"api/main/getprocedure?id={id}");
            return View(model);
        }
        [HttpPost]
        public void CreateProcedure(ProceduresBindingModel model, int? id, string procedurename, string proceduretype )
        {

            if (APIClient.Client == null)
            {
                throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
            }
            if (model.Id != 0)
            {
                APIClient.PostRequest("api/main/updateprocedure", new ProceduresBindingModel
                {
                    Id = id.Value,
                    ProceduresName = procedurename,
                    Type = proceduretype
                });
            }
            else
            {
                APIClient.PostRequest("api/main/createprocedure", new ProceduresBindingModel
                {
                    ClientId = APIClient.Client.Id,
                    ProceduresName = procedurename,
                    Type = proceduretype

                });
            }
            Response.Redirect("ListProcedures");
        }

        [HttpGet]
        public Tuple<ProceduresViewModel, string>? GetProcedure(int procedureId)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            var result = APIClient.GetRequest<Tuple<ProceduresViewModel, List<Tuple<string>>>>($"api/main/getprocedure?procedureId={procedureId}");
            if (result == null)
            {
                return default;
            }
            string table = "";
            for (int i = 0; i < result.Item2.Count; i++)
            {
                var medicinesName = result.Item2[i].Item1;
                table += "<tr style=\"height: 44px\">";
                table += $"<td class=\"u-border-1 u-border-grey-30 u-table-cell\">{medicinesName}</td>";
                table += "</tr>";
            }
            return Tuple.Create(result.Item1, table);
        }

        public IActionResult AddMedicineToProcedure()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(Tuple.Create(APIClient.GetRequest<List<ProceduresViewModel>>($"api/main/getprocedurelist?clientId={APIClient.Client.Id}"),
                APIClient.GetRequest<List<MedicinesViewModel>>($"api/main/getmedicinelist?clientId={APIClient.Client.Id}")));
        }

        [HttpPost]
        public void AddMedicineToProcedure(int procedure, int[] medicine)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            for (int i = 0; i < medicine.Length; i++)
            {
                APIClient.PostRequest("api/main/AddMedicineToProcedure", Tuple.Create(
                    new ProceduresSearchModel() { Id = procedure },
                    new MedicinesViewModel() { Id = medicine[i] }
                ));
            }
            Response.Redirect("ListConferences");
        }

        [HttpPost]
        public void DeleteProcedure(int procedure)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Доступно только авторизованным пользователям");
            }

            APIClient.PostRequest($"api/main/DeleteProcedure", new ProceduresBindingModel { Id = procedure });
            Response.Redirect("ListProcedures");
        }

        public IActionResult DeleteProcedure()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Procedures = APIClient.GetRequest<List<ProceduresViewModel>>($"api/main/getprocedurelist?clientId={APIClient.Client.Id}");
            return View();
        }

        public IActionResult UpdateProcedure()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Procedures = APIClient.GetRequest<List<ProceduresViewModel>>($"api/main/getprocedurelist?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void UpdateProcedure(int procedure, string procedurename, string type)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            if (string.IsNullOrEmpty(procedurename))
            {
                throw new Exception("Название не может быть пустым");
            }
            if (string.IsNullOrEmpty(type))
            {
                throw new Exception("Тип не может быть пустым");
            }
            APIClient.PostRequest("api/main/UpdateProcedure", new ProceduresBindingModel
            {
                Id = procedure,
                ProceduresName = procedurename,
                Type = type,
                ClientId = APIClient.Client.Id
            });
            Response.Redirect("ListProcedures");
        }

        /// <summary>
        /// ЛЕКАРСТВА
        /// </summary>
        /// <returns></returns>
        public IActionResult ListMedicines()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<MedicinesViewModel>>($"api/main/getmedicineslist?clientId={APIClient.Client.Id}"));
        }
        public IActionResult CreateMedicine()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View();
        }
        [HttpPost]
        public void CreateMedicine(string namemedicine, string group)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
            }
            APIClient.PostRequest("api/main/createmedicine", new MedicinesBindingModel
            {
                ClientId = APIClient.Client.Id,
                MedicinesName = namemedicine,
                Group = group
            });
            Response.Redirect("ListMedicines");
        }
        [HttpGet]
        public MedicinesViewModel? GetMedicine(int medicineId)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            var result = APIClient.GetRequest<MedicinesViewModel>($"api/main/getmedicine?medicineid={medicineId}");
            if (result == null)
            {
                return default;
            }
            var medicinename = result.MedicinesName;
            var group = result.Group;

            return result;
        }

        [HttpPost]
        public void DeleteMedicine(int medicine)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Доступно только авторизованным пользователям");
            }

            APIClient.PostRequest($"api/main/DeleteMedicine", new ProceduresBindingModel { Id = medicine });
            Response.Redirect("ListMedicines");
        }

        public IActionResult DeleteMedicine()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Medicines = APIClient.GetRequest<List<MedicinesViewModel>>($"api/main/GetMedicinesList?clientId={APIClient.Client.Id}");
            return View();
        }

        public IActionResult UpdateMedicine()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Medicines = APIClient.GetRequest<List<MedicinesViewModel>>($"api/main/getmedicineslist?clientId={APIClient.Client.Id}");
            return View();
        }

        [HttpPost]
        public void UpdateMedicine(int medicine, string medicinename, string group)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            if (string.IsNullOrEmpty(medicinename))
            {
                throw new Exception("Название не может быть пустым");
            }
            if (string.IsNullOrEmpty(group))
            {
                throw new Exception("Группа не может быть пустой");
            }
            APIClient.PostRequest("api/main/UpdateMedicine", new MedicinesBindingModel
            {
                Id = medicine,
                MedicinesName = medicinename,
                Group = group,
                ClientId = APIClient.Client.Id
            });
            Response.Redirect("ListMedicines");
        }
        /// <summary>
        /// РЕЦЕПТЫ
        /// </summary>
        /// <returns></returns>
        public IActionResult ListRecipes()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            return View(APIClient.GetRequest<List<RecipesViewModel>>($"api/main/getrecipeslist?clientId={APIClient.Client.Id}"));
        }
        public IActionResult CreateRecipe(int? id)
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Symptomses = APIClient.GetRequest<List<SymptomsViewModel>>("api/main/getsymptoms");
            ViewBag.Procedures = APIClient.GetRequest<List<ProceduresViewModel>>("api/main/getprocedurelist");
            if (!id.HasValue)
            {
                return View(new RecipesViewModel());
            }
           
            var model = APIClient.GetRequest<ProceduresViewModel?>($"api/main/getprocedure?id={id}");
            return View(model);

        }

        [HttpPost]
        public void CreateRecipe(RecipesBindingModel model, int? id, int symptoms, string dose, string modeofapplication)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Вы как сюда попали? Сюда вход только авторизованным");
            }
            model.ClientId = APIClient.Client.Id;
            if (model.Id != 0)
            {
                APIClient.PostRequest("api/main/updaterecipe", model);
            }
            else
            {
                APIClient.PostRequest("api/main/createrecipe", new RecipesBindingModel
                {
                    ClientId = APIClient.Client.Id,
                    Dose = dose,
                    SymptomsId = symptoms,
                    ModeOfApplication = modeofapplication
                });
            }
            Response.Redirect("ListRecipes");
        }

        [HttpGet]
        public Tuple<RecipesViewModel, string>? GetRecipe(int recipeId)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            var result = APIClient.GetRequest<Tuple<RecipesViewModel, List<Tuple<string>>>>($"api/main/getrecipe?recipeId={recipeId}");
            if (result == null)
            {
                return default;
            }
            string table = "";
            for (int i = 0; i < result.Item2.Count; i++)
            {
                var proceduresName = result.Item2[i].Item1;
                table += "<tr style=\"height: 44px\">";
                table += $"<td class=\"u-border-1 u-border-grey-30 u-table-cell\">{proceduresName}</td>";
                table += "</tr>";
            }
            return Tuple.Create(result.Item1, table);
        }
        [HttpPost]
        public void DeleteRecipe(int recipe)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Доступно только авторизованным пользователям");
            }

            APIClient.PostRequest($"api/main/DeleteRecipe", new RecipesBindingModel { Id = recipe });
            Response.Redirect("ListRecipes");
        }

        public IActionResult DeleteRecipe()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Recipes = APIClient.GetRequest<List<RecipesViewModel>>($"api/main/GetRecipesList?clientId={APIClient.Client.Id}");
            return View();
        }

        public IActionResult UpdateRecipe()
        {
            if (APIClient.Client == null)
            {
                return Redirect("~/Home/Enter");
            }
            ViewBag.Recipes = APIClient.GetRequest<List<RecipesViewModel>>($"api/main/GetRecipesList?clientId={APIClient.Client.Id}");
            ViewBag.Symptomses = APIClient.GetRequest<List<SymptomsViewModel>>("api/main/getsymptoms");
            return View();
        }

        [HttpPost]
        public void UpdateRecipe(int recipe, string dose, string modeofapplication)
        {
            if (APIClient.Client == null)
            {
                throw new Exception("Необходима авторизация");
            }
            if (string.IsNullOrEmpty(dose))
            {
                throw new Exception("Поле не может быть пустым");
            }
            if (string.IsNullOrEmpty(modeofapplication))
            {
                throw new Exception("Способ применения не может быть пустым");
            }
            APIClient.PostRequest("api/main/updaterecipe", new RecipesBindingModel
            {
                Id = recipe,
                Dose = dose,
                ModeOfApplication = modeofapplication,
                ClientId = APIClient.Client.Id
            });
            Response.Redirect("ListRecipes");
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}