﻿using HospitalContracts.BindingModels;
using HospitalContracts.SearchModels;
using HospitalContracts.StoragesContracts;
using HospitalContracts.ViewModels;
using HospitalDataBaseImplements.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HospitalDataBaseImplements.Implements
{
    public class KurseStorage : IKurseStorage
    {
        public KurseViewModel? Delete(KurseBindingModel model)
        {
            using var context = new HospitalDatabase();
            var element = context.Kurse.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Kurse.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }

        public KurseViewModel? GetElement(KurseSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new HospitalDatabase();
            return context.Kurse.Include(x => x.Medicines)
                .ThenInclude(x => x.Medicine).FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<KurseViewModel> GetFilteredList(KurseSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new HospitalDatabase();
            return context.Kurse.Include(x => x.Medicines)
                .ThenInclude(x => x.Medicine)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<KurseViewModel> GetFullList()
        {
            using var context = new HospitalDatabase();
            return context.Kurse.Include(x => x.Medicines)
                .ThenInclude(x => x.Medicine).Select(x => x.GetViewModel).ToList();
        }

        public KurseViewModel? Insert(KurseBindingModel model)
        {
            using var context = new HospitalDatabase();
            var newKurse = Kurses.Create(context, model);
            if (newKurse == null)
            {
                return null;
            }
            
            context.Kurse.Add(newKurse);
            context.SaveChanges();
            return context.Kurse
               .Include(x => x.Medicines)
               .ThenInclude(x => x.Medicine).FirstOrDefault(x => x.Id == newKurse.Id)
                         ?.GetViewModel;
        }

        public KurseViewModel? Update(KurseBindingModel model)
        {
            using var context = new HospitalDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var kurse = context.Kurse.FirstOrDefault(x => x.Id == model.Id);
                if (kurse == null)
                {
                    return null;
                }
                kurse.Update(model);
                context.SaveChanges();
                if (model.KurseMedicines != null)
                {
                    kurse.UpdateMedicines(context, model);
                }
                transaction.Commit();
                return kurse.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
    }
}
