﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HospitalDataModels.Models;
using HospitalContracts.BindingModels;
using HospitalContracts.ViewModels;
using System.Xml.Linq;

namespace HospitalDataBaseImplements.Models
{
    public class Kurses : IKurseModel
    {
    public int Id { get; private set; }
    [Required]
    public string Duration { get; private set; } = string.Empty;
    [Required]
    public int CountInDay { get; private set; }
    [ForeignKey("KurseId")]

    public virtual List<IllnessKurse> IllnessKurses { get; set; } = new();

    private Dictionary<int, IMedicinesModel>? _kurseMedicine = null;

        [NotMapped]
        public Dictionary<int, IMedicinesModel> KurseMedicines
        {
            get
            {
                using var context = new HospitalDatabase();
                if (_kurseMedicine == null)
                {
                    _kurseMedicine = Medicines
                    .ToDictionary(x => x.MedicineId, x =>
                   (context.Medicines.FirstOrDefault(y => y.Id == x.MedicineId)! as IMedicinesModel));
                }
                return _kurseMedicine;
            }
        }
        [ForeignKey("KurseId")]
        public virtual List<KurseMedicines> Medicines { get; set; } = new();
        public static Kurses? Create(HospitalDatabase context, XElement element)
    {
            if (element == null)
            {
                return null;
            }
            return new Kurses()
            {
                Duration = element.Element("Duration")!.Value,
                CountInDay = Convert.ToInt32(element.Element("CountInDay")!.Value),
                Medicines = element.Element("Medicines")!.Elements("MedicineId").Select(x => new KurseMedicines
                {
                    Medicine = context.Medicines.First(y => y.Id == Convert.ToInt32(x.Value))
                }).ToList()
            };
        }
    public static Kurses Create(HospitalDatabase context, KurseBindingModel model)
    {
        return new Kurses
        {
            Id = model.Id,
            Duration = model.Duration,
            CountInDay = model.CountInDay,
            Medicines = model.KurseMedicines.Select(x => new KurseMedicines
            {
                Medicine = context.Medicines.First(y => y.Id == x.Key)
            }).ToList()
        };
    }
    public void Update(KurseBindingModel model)
    {
        if (model == null)
        {
            return;
        }
        Duration = model.Duration;
        CountInDay = model.CountInDay;
    }
    public KurseViewModel GetViewModel => new()
    {
        Id = Id,
        Duration = Duration,
        CountInDay = CountInDay,
        KurseMedicines = KurseMedicines
    };
        public void UpdateMedicines(HospitalDatabase context, KurseBindingModel model)
        {
            var kurseMedicine = context.KurseMedicines.Where(rec => rec.KurseId == model.Id).ToList();
            if (kurseMedicine != null && kurseMedicine.Count > 0)
            {
                context.KurseMedicines.RemoveRange(kurseMedicine.Where(rec
               => !model.KurseMedicines.ContainsKey(rec.MedicineId)));
                context.SaveChanges();
            }
            var kurse = context.Kurse.First(x => x.Id == Id);
            var existingMedicineIds = kurseMedicine?.Select(x => x.MedicineId).ToList();
            foreach (var rec in model.KurseMedicines)
            {
                context.KurseMedicines.Add(new KurseMedicines
                {
                    Kurse = kurse,
                    Medicine = context.Medicines.First(x => x.Id == rec.Key),
                });
            }
            context.SaveChanges();
            _kurseMedicine = null;
        }
    }
}
