﻿using HospitalContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using HospitalContracts.SearchModels;
using HospitalContracts.BindingModels;

namespace HospitalView
{
    public partial class FormProcedure : Form
    {
        private readonly ILogger _logger;
        private readonly IProceduresLogic _logic;
        private int? _id;
        public int Id { set { _id = value; } }
        public FormProcedure(ILogger<FormProcedure> logger, IProceduresLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
        }
        private void FormProcedures_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                try
                {
                    _logger.LogInformation("Получение процедуры");
                    var view = _logic.ReadElement(new ProceduresSearchModel
                    {
                        Id = _id.Value
                    });
                    if (view != null)
                    {
                        textBoxNameProcedure.Text = view.ProceduresName;
                        textBoxTypeProcedure.Text = view.Type;
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка получения компонента");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }
        }
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxNameProcedure.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка",
               MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение процедуры");
            try
            {
                var model = new ProceduresBindingModel
                {
                    Id = _id ?? 0,
                    ProceduresName = textBoxNameProcedure.Text,
                    Type = textBoxTypeProcedure.Text
                };
                var operationResult = _id.HasValue ? _logic.Update(model) :
               _logic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение",
               MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения процедуры");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
               MessageBoxIcon.Error);
            }
        }
         private void ButtonCancel_Click(object sender, EventArgs e)
         {
             DialogResult = DialogResult.Cancel;
             Close();
         }
    }
}
